var Generator = require('yeoman-generator');
const yaml = require('js-yaml');
var path = require('path');

module.exports = class extends Generator{
  collect(){
    return this.prompt([
      {
        type    : 'input',
        name    : 'name',
        message : 'Your api name',
        default : 'api'
      },{
        type    : 'input',
        name    : 'port',
        message : 'Your api port',
        default : '8888'
      }
    ]).then(function (answers) {
      var cwd = this.destinationRoot()
      var project = cwd.split(path.sep).pop()
      this.api_name = answers.name
      this.api_port = answers.port

      // write this API into the docker compose file
      var f = "docker-compose.yaml"
      var root = {
        version: "3.8",
        services: {}
      }
      if(this.fs.exists(f)){
        root = yaml.load(this.fs.read(f))
      }
      root.services[this.api_name] = {
        volumes: ["./logs:/var/log","./storage:/storage","./"+this.api_name+":/app:ro","./local:/conf:ro"],
        image: project+"-"+this.api_name,
        environment: ["DSN=${DSN}"],
        ports: [this.api_port+":80"],
        "hostname": this.api_name+".local"
      }
      this.fs.write(f,yaml.dump(root))

      // write this API in local/apis.json
      var f = "local/apis.json"
      var apis = {}
      if(this.fs.exists(f)){
        apis = this.fs.readJSON(f)
      }

      apis[this.api_name] = {
        url: "http://"+this.api_name+".local/"+this.api_name+"/",
        delegates: ["DTProviderDelegate"],
        consumer_key: "abcde01234",
        consumer_secret: "abcde01234"
      }
      var api_url = "http://"+this.api_name+".local/"+this.api_name+"-oauth/"
      apis[this.api_name+"-oauth"] = {
        url: api_url,
        delegates: ["DTProviderDelegate","DTOAuth2Delegate"],
        consumer_key: "abcde01234",
        consumer_secret: "abcde01234",
        authorize_url: "login",
        redirect_uri: "http://localhost/"+this.api_name+"-oauth/",
        access_token_url: api_url+"user/access_token",
        response_type: "code",
        scope: "any"
      }
      this.fs.writeJSON(f,apis)
    }.bind(this));
  }

  install(){
    this.spawnCommandSync('mkdir',[this.api_name])
    var cwd = this.destinationRoot()
    var project = cwd.split(path.sep).pop()
    this.destinationRoot(this.api_name)
    this.spawnCommandSync('git',['init'])
    this.spawnCommandSync('git',['pull','https://gitlab.com/expressive-analytics/deep-thought/sample-api.git'])
    this.spawnCommandSync('git',['remote','add','origin','git@gitlab.com:expressive-analytics/'+project+'-'+this.api_name+'.git'])
    this.spawnCommandSync('git',['checkout','-b','developer'])
    this.spawnCommandSync('composer',['install','--no-dev'])
    this.destinationRoot(cwd)
  }
}
